/* This file is part of Mailfromd.             -*- c -*-
   Copyright (C) 2006, 2007, 2008, 2009, 2010 Sergey Poznyakoff

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

static int
valid_user_p(eval_environ_t env, const char *name)
{
	int rc;
	struct mu_auth_data *auth_data = NULL;

	rc = mu_get_auth(&auth_data, mu_auth_key_name, name);
	mu_auth_data_free(auth_data);
	switch (rc) {
	case 0:
		rc = 1;
		break;
		
	case MU_ERR_AUTH_FAILURE:
		rc = 0;
		break;

	case EAGAIN:
                MF_THROW(mfe_temp_failure,
		         _("temporary failure querying for username %s"),
			 name);
		break;

	default:
                MF_THROW(mfe_failure,
		         _("failure querying for username %s"),
			 name);
		break;
	}

	MF_DEBUG(10, 
                 ("Checking user %s: %s", name, rc ? "true" : "false"));
	return rc;
}

MF_DEFUN(validuser, NUMBER, STRING name)
{
	MF_RETURN(valid_user_p(env, name));
}
END

MF_DEFUN(interval, NUMBER, STRING str)
{
       time_t t;
       const char *endp;

       MF_ASSERT(parse_time_interval(str, &t, &endp) == 0,
	         mfe_invtime,
		 _("unrecognized time format (near `%s')"), endp);
       MF_RETURN(t);
}
END

MF_DEFUN(debug_level, NUMBER, OPTIONAL, STRING modname)
{
	int level;
	char *name = MF_OPTVAL(modname, NULL);
	size_t len = name ? 0 : strlen (name);
	
	MF_ASSERT(mu_debug_category_level(name, len, &level) == 0,
		  mfe_range,
		  _("invalid module name: %s"), name);
	MF_RETURN(level);
}
END

/* FIXME: minlevel is no longer used */
MF_DEFUN(debug_spec, STRING, OPTIONAL, STRING modnames, NUMBER minlevel,
	 NUMBER showunset)
{
	mu_stream_t str;
	size_t off;
	int rc;
	
	rc = mu_memory_stream_create(&str, MU_STREAM_RDWR);
	MF_ASSERT(rc == 0,
		  mfe_failure,
		  "cannot create stream: %s", mu_strerror(rc));
	rc = mu_debug_format_spec(str, MF_OPTVAL(modnames, NULL),
				  MF_OPTVAL(showunset, 0));
	if (rc == 0) {
		mu_off_t size;
		char *s;
		size_t n;
		
		mu_stream_seek(str, 0, MU_SEEK_SET, NULL);
		mu_stream_size(str, &size);
		s = MF_ALLOC_HEAP(off, size + 1);
		rc = mu_stream_read(str, s, size, &n);
		s[n] = 0;
	}
	
	mu_stream_destroy(&str);
	MF_ASSERT(rc == 0,
		  mfe_failure,
		  "%s", mu_strerror(rc));
	MF_RETURN(off);
}
END

MF_DEFUN(debug, VOID, STRING spec)
{
	mu_debug_parse_spec(spec);
}
END

MF_DEFUN(program_trace, VOID, STRING name)
{
	enable_prog_trace(name);
}
END

MF_DEFUN(cancel_program_trace, VOID, STRING name)
{
	disable_prog_trace(name);
}
END	

MF_DEFUN(stack_trace, VOID)
{
	runtime_stack_trace(env);
}
END

MF_INIT
